/* Virginia Tech Cognitive Radio Open Source Systems
 * Virginia Tech, 2009
 *
 * LICENSE INFORMATION GOES HERE
 */

/* DESCRIPTION OF FILE.
 */


#include <cstdlib>
#include <cstring>
#include <stdint.h>

#include "vtcross/common.h"
#include "vtcross/components.h"
#include "vtcross/containers.h"
#include "vtcross/debug.h"
#include "vtcross/error.h"
#include "vtcross/socketcomm.h"


PolicyEngine::PolicyEngine()
{
    LOG("Creating Policy Engine.\n");
    SML_present = false;
    LoadPolicies();
}


PolicyEngine::~PolicyEngine()
{
}


PolicyEngine::PolicyEngine(const char* serverName, const char* portNumber, \
        const bool SML)
{
    LOG("Creating Policy Engine.\n");

    SML_present = SML;

    commandSocketFD = ClientSocket(serverName, portNumber);

    if(SML_present) {
        RegisterServices();
        LOG("Policy Engine connected to SML at %s.\n", serverName);
    }
    else {
        RegisterComponent();
        LOG("Policy Engine connected to shell at %s.\n", serverName);
    }

    LoadPolicies();
}


void 
PolicyEngine::GetRemoteComponentType()
{
}


void 
PolicyEngine::WaitForSignal()
{
    LOG("Policy Engine:: Waiting for Policy Check Request.\n");

    int32_t decision_array[10];
    struct Parameter pList[10];
    struct CE_Info ce_info;

    // TODO should this be a GetRequest call or something else?
    if(GetRequest(commandSocketFD, pList, &ce_info)) {
        LOG("Policy Engine:: Validating Transmission Parameters.\n");
        ValidateParameters(pList, &ce_info, decision_array);    

        LOG("Policy Engine:: Sending Policy decision to Server.\n");
        SendPEDecision(pList, &ce_info, decision_array);
    }
}


void 
PolicyEngine::Shutdown()
{
    if(SML_present)
        DeregisterServices();
    else
        DeregisterComponent();
}


void
PolicyEngine::Reset()
{
    LOG("Resetting Policy Engine.\n");
    SML_present = false;
    commandSocketFD = -1;
    LoadPolicies();
}


void
PolicyEngine::RegisterComponent()
{
    SendMessage(commandSocketFD, "register_engine_policy");
    LOG("Policy Engine:: Registration message sent.\n");

}


void 
PolicyEngine::DeregisterComponent()
{
    SendMessage(commandSocketFD, "deregister_engine_policy");
    LOG("Policy Engine:: Registration message sent.\n");
}


void 
PolicyEngine::RegisterServices()
{
    LOG("Policy Engine:: Registering services.\n");
    SendMessage(commandSocketFD, "register_service_pe_geo");
    SendMessage(commandSocketFD, "register_service_pe_time");
    SendMessage(commandSocketFD, "register_service_pe_spectrum");
    SendMessage(commandSocketFD, "register_service_pe_spacial");
}


void 
PolicyEngine::DeregisterServices()
{
    LOG("Policy Engine:: Deregistering services.\n");
    SendMessage(commandSocketFD, "deregister_service_pe_geo");
    SendMessage(commandSocketFD, "deregister_service_pe_time");
    SendMessage(commandSocketFD, "deregister_service_pe_spectrum");
    SendMessage(commandSocketFD, "deregister_service_pe_spacial");
}


void 
PolicyEngine::LoadPolicies()
{
    LOG("PolicyEngine:: Loading policies.\n");
}


void 
PolicyEngine::ReloadPolicies()
{
    LOG("PolicyEngine:: Reloading policies.\n");
}


void 
PolicyEngine::SendPEDecision(struct Parameter pList[], \
        struct CE_Info *ce_info, int32_t decision_array[])
{
    char var[50];
 
    for (size_t i = 0; i < ce_info->numParameters; i++) {
        sprintf(var, "%i", decision_array[i]);
        SendMessage(commandSocketFD, var);
    }
}


void 
PolicyEngine::ValidateParameters(struct Parameter pList[], \
        struct CE_Info *ce_info, int decision_array[])
{
    LOG("Policy Engine:: Policies Validated.\n");
    for (size_t i = 0; i < ce_info->numParameters; i++)
        decision_array[i] = 1;
}

